import dns.resolver
from colorama import Fore, Style
from safemx.parsers import parse_spf_record, parse_dmarc_record, parse_dkim_record

def check_spf(domain, output_format='console'):
    spf_data = {}
    try:
        answers = dns.resolver.resolve(domain, 'TXT')
        for rdata in answers:
            record = str(rdata).strip('"')

            if 'v=spf1' in record:
                if output_format == 'console':
                    print(f"{Fore.GREEN}[+] SPF record for {domain} found!{Style.RESET_ALL}")
                    print(f"    spf: {record}\n")

                spf_data = parse_spf_record(record, output_format)
                return spf_data
        if output_format == 'console':
            print(f"{Fore.RED}[!] No SPF record found for {domain}{Style.RESET_ALL}! Careful! Attackers can send emails on behalf of this domain.")
        else:
            spf_data['error'] = f"No SPF record found for {domain}"
            spf_data['error_code'] = "NO_SPF"
    except dns.resolver.NoAnswer:
        if output_format == 'console':
            print(f"{Fore.RED}[!] No SPF record found for {domain}{Style.RESET_ALL}")
        else:
            spf_data['error'] = f"No SPF record found for {domain}"
            spf_data['error_code'] = "NO_SPF"
    except dns.resolver.NXDOMAIN:
        if output_format == 'console':
            print(f"{Fore.RED}[!] Domain {domain} does not exist.{Style.RESET_ALL}")
        else:
            spf_data['error'] = f"Domain {domain} does not exist."
            spf_data['error_code'] = "NXDOMAIN"
    except Exception as e:
        if output_format == 'console':
            print(f"{Fore.RED}[!] An error occurred while retrieving SPF: {e}{Style.RESET_ALL}")
        else:
            spf_data['error'] = f"An error occurred while retrieving SPF: {e}"
            spf_data['error_code'] = "UNKNOWN"
    return spf_data

def check_dmarc(domain, output_format='console'):
    dmarc_data = {}
    try:
        dmarc_domain = f"_dmarc.{domain}"
        answers = dns.resolver.resolve(dmarc_domain, 'TXT')
        for rdata in answers:
            record = str(rdata).strip('"')
            if 'v=DMARC1' in record:
                if output_format == 'console':
                    print(f"{Style.BRIGHT}{Fore.GREEN}[+] DMARC record for {domain} found!{Style.RESET_ALL}")
                    print(f"    dmarc: {record}\n")

                dmarc_data = parse_dmarc_record(record, output_format)
                return dmarc_data
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] No DMARC record found for {domain}{Style.RESET_ALL}")
        else:
            dmarc_data['error'] = f"No DMARC record found for {domain}"
            dmarc_data['error_code'] = "NO_DMARC"
    except dns.resolver.NoAnswer:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] No DMARC record found for {domain}{Style.RESET_ALL}")
        else:
            dmarc_data['error'] = f"No DMARC record found for {domain}"
            dmarc_data['error_code'] = "NO_DMARC"
    except dns.resolver.NXDOMAIN:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] Domain {domain} does not exist.{Style.RESET_ALL}")
        else:
            dmarc_data['error'] = f"Domain {domain} does not exist."
            dmarc_data['error_code'] = "NXDOMAIN"
    except Exception as e:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] An error occurred while retrieving DMARC: {e}{Style.RESET_ALL}")
        else:
            dmarc_data['error'] = f"An error occurred while retrieving DMARC: {e}"
            dmarc_data['error_code'] = "UNKNOWN"
    return dmarc_data

def check_dkim(domain, selector, output_format='console'):
    dkim_data = {}
    try:
        dkim_domain = f"{selector}._domainkey.{domain}"
        answers = dns.resolver.resolve(dkim_domain, 'TXT')
        for rdata in answers:
            record = str(rdata).strip('"')
            if 'v=DKIM1' in record:
                if output_format == 'console':
                    print(f"{Style.BRIGHT}{Fore.GREEN}[+] DKIM record for {domain} found with selector '{selector}'!{Style.RESET_ALL}")
                    print(f"    dkim: {record}\n")

                dkim_data = parse_dkim_record(record, output_format)
                return dkim_data
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] No DKIM record found for {domain} with selector '{selector}'{Style.RESET_ALL}")
        else:
            dkim_data['error'] = f"No DKIM record found for {domain} with selector '{selector}'"
            dkim_data['error_code'] = "NO_DKIM"
    except dns.resolver.NoAnswer:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] No DKIM record found for {domain} with selector '{selector}'{Style.RESET_ALL}")
        else:
            dkim_data['error'] = f"No DKIM record found for {domain} with selector '{selector}'"
            dkim_data['error_code'] = "NO_DKIM"
    except dns.resolver.NXDOMAIN:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] Domain {domain} does not exist.{Style.RESET_ALL}")
        else:
            dkim_data['error'] = f"Domain {domain} does not exist."
            dkim_data['error_code'] = "NXDOMAIN"
    except Exception as e:
        if output_format == 'console':
            print(f"{Style.BRIGHT}{Fore.RED}[!] An error occurred while retrieving DKIM: {e}{Style.RESET_ALL}")
        else:
            dkim_data['error'] = f"An error occurred while retrieving DKIM: {e}"
            dkim_data['error_code'] = "UNKNOWN"
    return dkim_data



from colorama import Fore, Style, init
from safemx.explanations import spf_tag_explanations, dmarc_tag_explanations, dkim_tag_explanations

def parse_spf_record(spf_record, output_format='console'):
    spf_parts = spf_record.split()
    spf_data = {
        'record': spf_record,
        'version': None,
        'mechanisms': [],
        'warnings': [],
        'notes': []
    }

    version_detected = False
    ip_detected = False
    include_detected = False
    all_detected = False
    redirect_detected = False

    if any(c.isupper() for c in spf_record):
        warning_msg = "SPF record contains uppercase characters, which is invalid."
        spf_data['warnings'].append(warning_msg)
        if output_format == 'console':
            print(f"{Fore.RED}[!] {warning_msg}{Style.RESET_ALL}")

    for part in spf_parts:
        if part.startswith("v="):
            version_detected = True
            spf_data['version'] = part
            if output_format == 'console':
                print(f"    {Style.BRIGHT}{Fore.CYAN}Version Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations['v']}")
        elif part.startswith("ip4:") or part.startswith("ip6:"):
            ip_detected = True
            spf_data['mechanisms'].append({'type': 'ip', 'value': part})
            if output_format == 'console':
                print(f"    {Style.BRIGHT}{Fore.GREEN}IP Address Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations[part.split(':')[0]]}")
        elif part.startswith("include:") or part.startswith("+include:"):
            include_detected = True
            domain = part.split(":", 1)[1]
            spf_data['mechanisms'].append({'type': 'include', 'value': part, 'domain': domain})
            if output_format == 'console':
                print(f"    {Style.BRIGHT}{Fore.YELLOW}Include Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations['include']}")
        elif part.startswith("redirect="):
            redirect_detected = True
            redirect_domain = part.split("=", 1)[1]
            spf_data['mechanisms'].append({'type': 'redirect', 'value': part, 'domain': redirect_domain})
            if output_format == 'console':
                print(f"    {Style.BRIGHT}{Fore.YELLOW}Redirect Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations['redirect']}")
        elif part in ["-all", "~all", "?all", "+all"]:
            all_detected = True
            spf_data['mechanisms'].append({'type': 'all', 'value': part})
            if output_format == 'console':
                all_type = spf_tag_explanations['all'].get(part, f"Unknown 'all' type: {part}")
                print(f"    {Style.BRIGHT}{Fore.MAGENTA}{all_type} Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations['all'][part]}")
        else:
            spf_data['mechanisms'].append({'type': 'other', 'value': part})
            if output_format == 'console':
                print(f"    {Style.BRIGHT}{Fore.BLUE}Other Mechanism Detected:{Style.RESET_ALL} {part}")
                print(f"        [i] {spf_tag_explanations.get(part.split(':')[0], 'Unknown mechanism')}")

    if not version_detected:
        warning_msg = "SPF version not detected. This SPF record may be invalid without 'v=spf1'."
        spf_data['warnings'].append(warning_msg)
        if output_format == 'console':
            print(f"{Fore.RED}[!] {warning_msg}{Style.RESET_ALL}")
    
    if not ip_detected and not include_detected:
        warning_msg = "No IP or Include mechanisms detected. Your SPF record may not properly authorize any servers."
        spf_data['warnings'].append(warning_msg)
        if output_format == 'console':
            print(f"{Fore.RED}[!] {warning_msg}{Style.RESET_ALL}")

    if not all_detected and not redirect_detected:
        note_msg = "No 'all' or 'redirect=' mechanism detected. It's recommended to end SPF records with one of these."
        spf_data['notes'].append(note_msg)
        if output_format == 'console':
            print(f"{Fore.YELLOW}[!] {note_msg}{Style.RESET_ALL}")

    if output_format == 'console':
        print(f"{Style.BRIGHT}{Fore.CYAN}SPF Record Analysis Complete.{Style.RESET_ALL}")
    
    return spf_data

def parse_dmarc_record(dmarc_record, output_format='console'):
    dmarc_parts = dmarc_record.split(';')
    dmarc_data = {
        'record': dmarc_record,
        'fields': {},
        'warnings': [],
        'notes': []
    }

    for part in dmarc_parts:
        key_value = part.strip().split('=')
        if len(key_value) == 2:
            key, value = key_value[0].strip(), key_value[1].strip()
            dmarc_data['fields'][key] = value

            if output_format == 'console':
                if key == 'v':
                    print(f"    {Style.BRIGHT}{Fore.CYAN}Version Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {dmarc_tag_explanations['v']}")
                elif key == 'p':
                    policy_explanation = dmarc_tag_explanations['p'].get(value, f"Unknown policy: {value}")
                    print(f"    {Style.BRIGHT}{Fore.GREEN}Policy Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {policy_explanation}")
                elif key == 'adkim':
                    alignment_explanation = dmarc_tag_explanations['adkim'].get(value, f"Unknown alignment: {value}")
                    print(f"    {Style.BRIGHT}{Fore.YELLOW}DKIM Alignment Mode Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {alignment_explanation}")
                elif key == 'aspf':
                    alignment_explanation = dmarc_tag_explanations['aspf'].get(value, f"Unknown alignment: {value}")
                    print(f"    {Style.BRIGHT}{Fore.YELLOW}SPF Alignment Mode Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {alignment_explanation}")
                elif key == 'sp':
                    sp_explanation = dmarc_tag_explanations['sp'].get(value, f"Unknown subdomain policy: {value}")
                    print(f"    {Style.BRIGHT}{Fore.GREEN}Subdomain Policy Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {sp_explanation}")
                elif key == 'fo':
                    fo_explanation = dmarc_tag_explanations['fo'].get(value, f"Unknown forensic reporting option: {value}")
                    print(f"    {Style.BRIGHT}{Fore.MAGENTA}Forensic Options Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {fo_explanation}")
                elif key == 'ruf':
                    print(f"    {Style.BRIGHT}{Fore.BLUE}Forensic Reports URI Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {dmarc_tag_explanations['ruf']}")
                elif key == 'rua':
                    print(f"    {Style.BRIGHT}{Fore.BLUE}Aggregate Reports URI Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {dmarc_tag_explanations['rua']}")
                elif key == 'rf':
                    rf_explanation = dmarc_tag_explanations['rf'].get(value, f"Unknown report format: {value}")
                    print(f"    {Style.BRIGHT}{Fore.BLUE}Reporting Format Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {rf_explanation}")
                elif key == 'pct':
                    print(f"    {Style.BRIGHT}{Fore.MAGENTA}Percentage Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] {dmarc_tag_explanations['pct']} {value}% of emails will be subject to DMARC filtering.")
                elif key == 'ri':
                    print(f"    {Style.BRIGHT}{Fore.MAGENTA}Reporting Interval Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"    [i] Reporting interval is {value} seconds (default is 86400 seconds = 1 day).")

    if output_format == 'console':
        print(f"{Style.BRIGHT}{Fore.CYAN}DMARC Record Analysis Complete.{Style.RESET_ALL}")
    
    return dmarc_data

def parse_dkim_record(dkim_record, output_format='console'):
    dkim_parts = dkim_record.split(';')
    dkim_data = {
        'record': dkim_record,
        'fields': {},
        'warnings': [],
        'notes': []
    }

    for part in dkim_parts:
        key_value = part.strip().split('=')
        if len(key_value) == 2:
            key, value = key_value[0].strip(), key_value[1].strip()
            dkim_data['fields'][key] = value

            if output_format == 'console':
                if key == 'v':
                    print(f"    {Style.BRIGHT}{Fore.CYAN}Version Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"        [i] {dkim_tag_explanations['v']}")
                elif key == 'p':
                    print(f"    {Style.BRIGHT}{Fore.GREEN}Public Key Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"        [i] {dkim_tag_explanations['p']}")
                else:
                    print(f"    {Style.BRIGHT}{Fore.BLUE}Other Mechanism Detected:{Style.RESET_ALL} {key}={value}")
                    print(f"        [i] Unknown or unhandled DKIM mechanism.")

    if output_format == 'console':
        print(f"{Style.BRIGHT}{Fore.CYAN}DKIM Record Analysis Complete.{Style.RESET_ALL}")
    
    return dkim_data



import argparse
import sys
from safemx.checkers import check_spf, check_dmarc, check_dkim
from colorama import Fore, Style, init
import json

init(autoreset=True)

def display_banner():
    banner = r"""
  ________     __       _______   _______  ___      ___  ___  ___  
 /"       )   /""\     /"     "| /"     "||"  \    /"  ||"  \/"  | 
(:   \___/   /    \   (: ______)(: ______) \   \  //   | \   \  /  
 \___  \    /' /\  \   \/    |   \/    |   /\\  \/.    |  \\  \/   
  __/  \\  //  __'  \  // ___)   // ___)_ |: \.        |  /\.  \   
 /" \   :)/   /  \\  \(:  (     (:      "||.  \    /:  | /  \   \  
(_______/(___/    \___)\__/      \_______)|___|\__/|___||___/\___| 
                                                                   
    """
    print(Fore.MAGENTA + banner + Style.RESET_ALL)

def main():
    parser = argparse.ArgumentParser(
        description="Check domain's SPF, DMARC, and DKIM records.",
        usage='main.py [-h] [-spf] [-dmarc] [-dkim] [-selector SELECTOR] [--output OUTPUT] domain'
    )

    parser.add_argument('domain', help="The domain to check")
    parser.add_argument('-spf', action='store_true', help="Check SPF record")
    parser.add_argument('-dmarc', action='store_true', help="Check DMARC record")
    parser.add_argument('-dkim', action='store_true', help="Check DKIM record")
    parser.add_argument('-selector', type=str, help="DKIM selector for DKIM check")
    parser.add_argument('--output', choices=['console', 'json'], default='console', help='Output format')
    parser.add_argument('--outfile', type=str, help="Output file for JSON results", default='output.json')

    args = parser.parse_args()

    display_banner()

    if not any([args.spf, args.dmarc, args.dkim]):
        print("No check option provided, use -spf, -dmarc, or -dkim flags.")
        parser.print_help()
        sys.exit(1)

    output_data = {}

    if args.spf:
        spf_data = check_spf(args.domain, args.output)
        if args.output == 'json':
            output_data['spf'] = spf_data

    if args.dmarc:
        dmarc_data = check_dmarc(args.domain, args.output)
        if args.output == 'json':
            output_data['dmarc'] = dmarc_data

    if args.dkim:
        if args.selector is None:
            print(f"{Style.BRIGHT}{Fore.YELLOW}[!] No DKIM selector provided. Proceeding with default selector 'default'{Style.RESET_ALL}")
            args.selector = 'default'
        dkim_data = check_dkim(args.domain, args.selector, args.output)
        if args.output == 'json':
            output_data['dkim'] = dkim_data

    if args.output == 'json':
        with open(args.outfile, 'w') as json_file:
            json.dump(output_data, json_file, indent=4)
        print(f"JSON output written to {args.outfile}")

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="safemx",
    version="0.1.2",
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'safemx=safemx.main:main',
        ],
    },
    install_requires=[
        'argparse',
        'colorama',
        'dnspython'
    ],
    description="A tool to check domain's SPF, DMARC, and DKIM records.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="Marko Gordic",
    author_email="marko@gordic.rs",
    url="https://github.com/MarkoGordic/safemx",
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
